/* eslint-disable @typescript-eslint/ban-ts-comment */
import { Request, Response } from "express";
import httpStatus from "http-status";
import catchAsync from "../../../shared/catchAsync";
import pick from "../../../shared/pick";
import sendResponse from "../../../shared/sendResponse";
import { SubCategoryService } from "./subCategory.service";
import { SubCategoryFilterableFields } from "./subCategory.constants";

// !----------------------------------Create New Category---------------------------------------->>>
const addSubCategoryController = catchAsync(async (req: Request, res: Response) => {
  // @ts-ignore
  const result = await SubCategoryService.addSubCategory(req);

  sendResponse(res, {
    statusCode: httpStatus.OK,
    success: true,
    message: "Sub Category Added Successfully",
    data: result,
  });
});

// !----------------------------------get all Category---------------------------------------->>>
const getSubCategoryController = catchAsync(async (req: Request, res: Response) => {
  const filters = pick(req.query, SubCategoryFilterableFields);

  const options = pick(req.query, ["limit", "page", "sortBy", "sortOrder"]);

  const result = await SubCategoryService.getSubCategory(filters, options);

  sendResponse(res, {
    statusCode: httpStatus.OK,
    success: true,
    message: "All Sub Categories fetched successfully",
    meta: result.meta,
    data: result.data,
  });
});

// !----------------------------------get Single Category---------------------------------------->>>
const getSingleSubCategory = catchAsync(async (req: Request, res: Response) => {
  const { subCategoryId } = req.params;
  const result = await SubCategoryService.getSingleSubCategory(subCategoryId);

  sendResponse(res, {
    statusCode: httpStatus.OK,
    success: true,
    message: "Sub Category retrieved successfully",
    data: result,
  });
});

// !----------------------------------get Single Category---------------------------------------->>>
const getSingleSubCategoryByName = catchAsync(async (req: Request, res: Response) => {
  const { subCategoryHref } = req.params;
  const result = await SubCategoryService.getSingleSubCategoryByName(subCategoryHref);

  sendResponse(res, {
    statusCode: httpStatus.OK,
    success: true,
    message: "Sub Category retrieved successfully",
    data: result,
  });
});

// !----------------------------------Update Category---------------------------------------->>>
const updateSubCategory = catchAsync(async (req: Request, res: Response) => {
  const { subCategoryId } = req.params;
  // @ts-ignore
  const result = await SubCategoryService.updateSubCategory(subCategoryId, req);

  sendResponse(res, {
    statusCode: httpStatus.OK,
    success: true,
    message: " Sub Category Updated successfully !",
    data: result,
  });
});

const deleteSubCategory = catchAsync(async (req: Request, res: Response) => {
  const { subCategoryId } = req.params;
  const result = await SubCategoryService.deleteSubCategory(subCategoryId);

  sendResponse(res, {
    statusCode: httpStatus.OK,
    success: true,
    message: "Sub Category Deleted successfully !",
    data: result,
  });
});

export const SubCategoryController = {
  addSubCategoryController,
  getSubCategoryController,
  getSingleSubCategory,
  updateSubCategory,
  deleteSubCategory,
  getSingleSubCategoryByName,
};
