import express, { NextFunction, Request, Response } from "express";

import { SubCategoryController } from "./subCategory.controller";
import { FileUploadHelper } from "../../../helpers/FileUploadHelper";
import auth from "../../middlewares/auth";
import { UserRoles } from "@prisma/client";
import { SubCategoryValidation } from "./subCategory.validation";

const router = express.Router();

// ! Create New List ------------------------------->>>

router.post(
  "/",
  // auth(UserRoles.USER, UserRoles.ADMIN, UserRoles.SUPER_ADMIN),
  FileUploadHelper.uploadSubCategoryImage.single("file"),
  (req: Request, res: Response, next: NextFunction) => {
    console.log("req.body", req.body);

    req.body = SubCategoryValidation.addSubCategory.parse(JSON.parse(req.body.data));
    return SubCategoryController.addSubCategoryController(req, res, next);
  },
);

// ! Get all List----------------------------------->>>
router.get(
  "/",
  // auth(UserRoles.USER, UserRoles.ADMIN, UserRoles.SUPER_ADMIN),
  SubCategoryController.getSubCategoryController,
);

router.get(
  "/id/:subCategoryId",
  // auth(UserRoles.USER, UserRoles.ADMIN, UserRoles.SUPER_ADMIN),
  SubCategoryController.getSingleSubCategory,
);

router.get(
  "/href/:subCategoryHref",
  // auth(UserRoles.USER, UserRoles.ADMIN, UserRoles.SUPER_ADMIN),
  SubCategoryController.getSingleSubCategoryByName,
);

router.patch(
  "/:subCategoryId",
  auth(UserRoles.SUPER_ADMIN),
  FileUploadHelper.uploadSubCategoryImage.single("file"),
  (req: Request, res: Response, next: NextFunction) => {
    req.body = SubCategoryValidation.editSubCategory.parse(JSON.parse(req.body.data));

    // JSON.parse(req.body.data);
    return SubCategoryController.updateSubCategory(req, res, next);
  },
);

router.delete(
  "/:subCategoryId",
  // auth(UserRoles.USER, UserRoles.ADMIN, UserRoles.SUPER_ADMIN),
  SubCategoryController.deleteSubCategory,
);

export const SubCategoryRoutes = router;
