import { UserRoles } from "@prisma/client";
import express from "express";
import auth from "../../middlewares/auth";
import validateRequest from "../../middlewares/validateRequest";
import { UserValidation } from "./user.validations";
import { UserController } from "./user.controller";

const router = express.Router();

// !  get all Users ------------------------------>>>
router.get("/", auth(UserRoles.SUPER_ADMIN), UserController.getAllUsersController);
// !  get client Users ------------------------------>>>
router.get("/get-clients", auth(UserRoles.SUPER_ADMIN), UserController.getAllClients);

// !  get My Profile ------------------------------>>>
router.get("/my-profile", auth(UserRoles.SUPER_ADMIN, UserRoles.USER), UserController.getMyProfile);

router.delete("/:userId", auth(UserRoles.SUPER_ADMIN), UserController.deleteUser);

// !  Update  my profile ------------------------------>>>
router.patch(
  "/update-my-profile",
  auth(UserRoles.SUPER_ADMIN, UserRoles.USER),
  validateRequest(UserValidation.updateMyProfile),
  UserController.updateMyProfileInfo,
);

// !  Update email and password ------------------------------>>>
router.patch(
  "/update-user/:userId",
  auth(UserRoles.SUPER_ADMIN),
  validateRequest(UserValidation.updateUser),
  UserController.updateUser,
);

// !  get single user ------------------------------>>>
router.get("/get-single-user/:userId", auth(UserRoles.SUPER_ADMIN), UserController.getSingleUser);

export const UserRoutes = router;
